local addonName, addon = ...

local plugin = addon:NewModule("LFD", "AceHook-3.0")
plugin.Icon = [[Interface\LFGFrame\BattlenetWorking19]]

local AceGUI	= LibStub("AceGUI-3.0")
local FA		= LibStub("LibFrameAnchorRegistry-1.0")

local db

local SettingPoistion = false
local ConfigMode = false

function plugin:OnInitialize()
	self.DefaultState = false
	
	local defaults = {
		profile = {
			Enabled = true,
			
			Anchor = "mMinimapTrackingButton",
			Point = "TOPLEFT",
			RelPoint = "BOTTOMLEFT",
			x = 0,
			y = 0,
			
			Scale = 1,
			
			Hide = false,
			ShowOnMouseOver = true,
			
			MoveLFGStatus = true,
		}
	}

	self.db = addon.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	FA:RegisterAnchor("mMinimap", "LFD Button", "mMinimapLFDButton", QueueStatusMinimapButton)
end

function plugin:OnEnable()	
	self:SecureHook(QueueStatusMinimapButton, "SetPoint", "SetPosition")
	self:HookScript(QueueStatusMinimapButton, "OnEnter", function()
		Minimap:GetScript("OnEnter")(Minimap)
	end)
	self:HookScript(QueueStatusMinimapButton, "OnLeave", function()
		Minimap:GetScript("OnLeave")(Minimap)
	end)
	
	addon.RegisterCallback(self, "MinimapEnter")
	addon.RegisterCallback(self, "MinimapLeave")
	addon.RegisterCallback(self, "ConfigMode")
	
	self:ApplySettings()
end

function plugin:OnDisable()
end

function plugin:ProfileChanged(event)
	db = self.db.profile
	self:ApplySettings()
end

function plugin:MinimapEnter()
	if not QueueStatusMinimapButton:IsShown() and db.ShowOnMouseOver and GetLFGMode(1) then
		QueueStatusMinimapButton:Show()
	end
end

function plugin:MinimapLeave()
	if QueueStatusMinimapButton:IsShown() and db.Hide and not ConfigMode then
		QueueStatusMinimapButton:Hide()
	end
end

function plugin:SetPosition(frame)
	if SettingPoistion then return end
	SettingPoistion = true
	frame:SetScale(db.Scale)
	frame:ClearAllPoints()
	local anchor = FA:GetAnchor(db.Anchor)
	local s = frame:GetEffectiveScale()
	frame:SetPoint(db.Point, anchor, db.RelPoint, db.x / s, db.y / s)
	SettingPoistion = false
end

function plugin:ApplySettings()	

	QueueStatusMinimapButton:SetPoint(db.Point)	
	QueueStatusMinimapButtonBorder:Hide()
	
	if db.MoveLFGStatus then
		QueueStatusFrame:ClearAllPoints()
		QueueStatusFrame:SetPoint("CENTER", UIParent)
	else
		QueueStatusFrame:ClearAllPoints()
		QueueStatusFrame:SetPoint("RIGHT", QueueStatusMinimapButton, "LEFT")
	end	
	
	if (db.Hide --[[or not GetLFGMode()]]) and not ConfigMode then
		QueueStatusMinimapButton:Hide()
	else
		QueueStatusMinimapButton:Show()
	end
end

function plugin:ConfigMode(event, enable)
	ConfigMode = enable
	if enable then
		QueueStatusMinimapButton:Show()
		EyeTemplate_StartAnimating(QueueStatusMinimapButton.eye)
	elseif db.Hide or not GetLFGMode() then
		QueueStatusMinimapButton:Hide()
		EyeTemplate_StopAnimating(QueueStatusMinimapButton.eye)
	end
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	function plugin:GetOptions()
		local g = AceGUI:Create("SimpleGroup")
		g:SetFullWidth(true)
		g:SetLayout("Flow")
		
		g:AddChild(UIF:CheckBox("Hide", db, "Hide", Callback, "Hide the LFD icon", 0.33))
		g:AddChild(UIF:CheckBox("Show on mouse over", db, "ShowOnMouseOver", Callback, "If hidden, the LFD icon will show up when hovering over the minimap", 0.33))
		g:AddChild(UIF:CheckBox("Move LFD Status", db, "MoveLFGStatus", Callback, "Moves the popup window when hovering over the LFD icon", 0.33))
		
		g:AddChild(UIF:NewLine())
		
		local t = AceGUI:Create("FrameAnchorDropdown")
		t:SetRelativeWidth(0.5)
		t:SetList(FA:GetAnchors(QueueStatusMinimapButton))
		t:SetLabel("Anchor")
		t:SetValue(db.Anchor)
		t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
			db.x = 0
			db.y = 0
			db.Anchor = cat
			Callback()
		end)
		g:AddChild(t)
		g:AddChild(UIF:Slider("Scale", db, "Scale", 0.1, 5, 0.01, Callback, 0.5))
		g:AddChild(UIF:GridSelect("Point", db, "Point", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:GridSelect("Relative Point", db, "RelPoint", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.44))
		return g
	end
end